@@ -135,7 +135,7 @@ class KustoReader:
         if self.is_table_exist(kusto_table_name):
             self.drop_table(kusto_table_name)
         
-        df          = pd.read_csv(csv_file_path)
+        df          = pd.read_csv(csv_file_path,nrows=3)    # nrows = 3 can avoid read massive csv file and blow up memory
         columns     = list(df.head())
         columns     = [c+":string" for c in columns]
         columns_str = str(columns).replace('[','').replace(']','').replace("'",'')
@@ -214,6 +214,9 @@ import os
 import sys
 
 class CosmosReader:
+    '''
+    The Cosmos Reader can only be used inside of Microsoft Corp Network
+    '''
     def __init__(self,scope_exe_path,client_account,vc_path) -> None:
         '''
         Initialize the CosmosReader object.
@@ -236,7 +239,7 @@ class CosmosReader:
         # check job status
         return output_str
     
-    def check_job_status(self,output_str,check_times = 30,check_gap_min=2):
+    def check_job_status(self,output_str,check_times = 60,check_gap_min=2):
         op_guid = output_str.split(',')[-2].split('\\r\\n')[0].split(':')[1].strip()
         print('op_guid:',op_guid)
         if len(op_guid) > 40:
@@ -366,8 +369,8 @@ class AzureBlobReader:
     def __init__(self,blob_conn_str,container_name) -> None:
         self.connect_string         = blob_conn_str
         self.blob_service_client    = BlobServiceClient.from_connection_string(self.connect_string)
-        self.blob_service_client.max_single_put_size = 4*1024*1024              # 4M
-        self.blob_service_client.timeout = 60*20                                # 10 mins
+        #self.blob_service_client.max_single_put_size = 4*1024*1024              # 4M
+        #self.blob_service_client.timeout = 60*20                                # 10 mins
         self.container_client       = self.blob_service_client.get_container_client(container_name)
 
     def download_file(self,blob_file_path,local_file_path) -> str:
@@ -382,7 +385,7 @@ class AzureBlobReader:
 
     def upload_file(self,blob_file_path,local_file_path):
         '''
-        Upload a local file to blob storage
+        Upload a local small file (< 9mb) to blob storage
         '''
         try:
             blob_client = self.container_client.get_blob_client(blob_file_path)
@@ -397,11 +400,14 @@ class AzureBlobReader:
             print(err)
     
     def upload_file_chunks(self,blob_file_path,local_file_path):
+        '''
+        Upload large file to blob
+        '''
         try:
             blob_client = self.container_client.get_blob_client(blob_file_path)
             # upload data
             block_list=[]
-            chunk_size=1024
+            chunk_size=1024*1024*4
             with open(local_file_path,'rb') as f:
                 while True:
                     read_data = f.read(chunk_size)
@@ -412,7 +418,7 @@ class AzureBlobReader:
                     block_list.append(BlobBlock(block_id=blk_id))
             blob_client.commit_block_list(block_list)
         except BaseException as err:
-            print('Upload file error')    
+            print('Upload file error')
             print(err)
 
     def get_blob_sas_token(self,expire_days = 1):
@@ -473,62 +479,65 @@ class Pipelines:
         ,kusto_target_table_name:str
         ,kusto_target_folder_name:str
     ):
-        # extract data from cosmos to local csv
-        cr  = CosmosReader(
-            scope_exe_path  = scope_exe_path
-            ,client_account = account
-            ,vc_path        = vc_path
-        )
-        r   = cr.scope_query(
-            scope_script        = scope_script
-            ,temp_data_path     = vc_temp_file_path
-            ,temp_query_data    = local_csv_file_path
-        )
-        print('data returned from cosmos:',r)
+        try:
+            # extract data from cosmos to local csv
+            cr  = CosmosReader(
+                scope_exe_path  = scope_exe_path
+                ,client_account = account
+                ,vc_path        = vc_path
+            )
+            r   = cr.scope_query(
+                scope_script        = scope_script
+                ,temp_data_path     = vc_temp_file_path
+                ,temp_query_data    = local_csv_file_path
+            )
+            print('data returned from cosmos:',r)
 
-        # upload to Azure blob container
-        abr = AzureBlobReader(
-            blob_conn_str   = blob_connect_str
-            ,container_name = blob_container
-        )
-        abr.upload_file(
-            blob_file_path  = blob_file_path
-            ,local_file_path= local_csv_file_path
-        )
-        sas_url = abr.get_blob_sas_url(
-            blob_file_path  = blob_file_path
-        )
-        print(f'local data {local_csv_file_path} uploaded to Azure Blob {blob_file_path}')
-        
-        # ingest to kusto
-        kr = KustoReader(
-            cluster             = kusto_cluster
-            ,db                 = kusto_db
-            ,ingest_cluster_str = kusto_ingest_cluster
-        )
-        kr.create_table_from_csv(
-            kusto_table_name    = kusto_target_table_name
-            ,csv_file_path      = local_csv_file_path
-            ,kusto_folder       = kusto_target_folder_name 
-        )
-        print(f'kusto table {kusto_target_table_name} is createed based on the csv file {local_csv_file_path}')
-        kr.upload_csv_from_blob(
-            target_table_name   = kusto_target_table_name
-            ,blob_sas_url       = sas_url
-        )
-        kr.check_table_data(kusto_target_table_name)
+            # upload to Azure blob container
+            abr = AzureBlobReader(
+                blob_conn_str   = blob_connect_str
+                ,container_name = blob_container
+            )
+            abr.upload_file_chunks(
+                blob_file_path  = blob_file_path
+                ,local_file_path= local_csv_file_path
+            )
+            sas_url = abr.get_blob_sas_url(
+                blob_file_path  = blob_file_path
+            )
+            print(f'local data {local_csv_file_path} uploaded to Azure Blob {blob_file_path}')
+            
+            # ingest to kusto
+            kr = KustoReader(
+                cluster             = kusto_cluster
+                ,db                 = kusto_db
+                ,ingest_cluster_str = kusto_ingest_cluster
+            )
+            kr.create_table_from_csv(
+                kusto_table_name    = kusto_target_table_name
+                ,csv_file_path      = local_csv_file_path
+                ,kusto_folder       = kusto_target_folder_name 
+            )
+            print(f'kusto table {kusto_target_table_name} is createed based on the csv file {local_csv_file_path}')
+            kr.upload_csv_from_blob(
+                target_table_name   = kusto_target_table_name
+                ,blob_sas_url       = sas_url
+            )
+            kr.check_table_data(kusto_target_table_name)
+
+        finally:
+            # clean up files
+            # # clean up local csv 
+            import os
+            os.remove('temp.script')
+            os.remove(local_csv_file_path)
+            # # clean up cosmos csv
+            cr.delete_file_from_cosmos(vc_temp_file_path)
+            # # delete file from blob
+            abr.delete_blob_file(blob_file_path)
+            
+            print('all temp data cleared')
 
-        # clean up files
-        # # clean up local csv 
-        import os
-        os.remove('temp.script')
-        os.remove(local_csv_file_path)
-        # # clean up cosmos csv
-        cr.delete_file_from_cosmos(vc_temp_file_path)
-        # # delete file from blob
-        abr.delete_blob_file(blob_file_path)
-        
-        print('all temp data cleared')
         print('all done')
 
 # endregion
\ No newline at end of file
