@@ -42,8 +42,8 @@ class vis_tools:
         self.hfont                      = {'fontname':'Helvetica'}
         self.msfont                     = {'fontname':'Segoe UI'}
         self.msfont_light               = {'fontname':'Segoe UI Light'}
-        self.label_text_font            = {'size':'20','weight':'bold','fontname':'Segoe UI Light'}
-        self.small_label_font           = {'size':'18','weight':'normal','fontname':'Segoe UI'}
+        self.label_text_font            = {'size':'20','weight':'bold'}
+        self.small_label_font           = {'size':'18','weight':'normal'}
         self.colors_seq                 = ['#003366','#A8C5E0','green','lime','sienna','violet']
         self.title_size                 = 20 
         self.axis_size                  = 18 
@@ -200,7 +200,7 @@ class vis_tools:
         ax.plot(x_label_index,y1_list,color=self.dark_blue)
         ax.plot(x_label_index,y2_list,color=self.light_blue)
         plt.xticks(x_label_index)
-        ax.set_xticklabels(x_list,**self.msfont_light)
+        ax.set_xticklabels(x_list)
         ax.set_ylim(bottom=0)
 
         ax.text(max(x_label_index),y1_list[-1],line1_name,**self.label_text_font,color=self.dark_blue)
